#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/usb.h>

typedef unsigned char __u8;
typedef unsigned short __u16;

typedef struct {
        __u8 requesttype;
        __u8 request;
        __u16 value;
        __u16 index;
        __u16 length;
} devrequest __attribute__ ((packed));

typedef unsigned char uchar;
#include "quickcamex.h"
#define EMPTY_TAG 0xaa

/*-----------------------------------------------------------------*/
// Read STV-register
int quick_reg_in(int fd, int reg)
{
	ctrl_transfer_t ct;

	ct.setup.requesttype=0xc0;
	ct.setup.request=0x04;
	ct.setup.value=reg;
	ct.setup.index=0;
	ct.size=1;

	ct.data[0] = EMPTY_TAG;

	ioctl(fd,IOCTL_QUICKCAM_CTRLR,&ct);
	return ct.data[0];
}

/*-----------------------------------------------------------------*/
int main(int argc, char** argv)
{
	int fd;
	int i,j;

	// Open the driver...
	fd=open("/dev/quick",O_RDWR);
	if (fd==-1)
	{
		perror("/dev/quick major 180, minor 80");
		exit(0);
	}


	printf("      "
	       "  0  1  2  3  4  5  6  7   8  9  A  B  C  D  E  F\n");
	// Read in all STV registers.

	for (i = 0; i < 0x10000; i+=16) {
	  int printed = 0;
	  /* Omit time consuming empty ranges */
	  if (i < 0x400
	      || (i > 0x420 && i < 0x540)
	      || (i > 0x560 && i < 0x1400)
	      || (i > 0x1700 && i < 0xb000)
	      || (i > 0xb000/* && i < 0xe000*/))
	    continue;
	  for (j = 0; j< 16; j++) {
	    __u8 val = quick_reg_in(fd, i+j);
	    if (val != EMPTY_TAG)
	      {
		if (!printed) {
		  int k;
		  printf("%4x:  ", i);
		  for (k=0; k< j; k++) {
		    if (k == 8)
		      printf(" ");
		    printf("   ");
		  }
		  printed=1;
		}
		if (j == 8)
		  printf(" ");
		printf(" %02x", val);
	      }
	    else if (printed)
	      {
		if (j == 8)
		  printf(" ");
		printf("   ");
	      }
	  }
	  if (printed)
	    printf("\n");
	}
	return 0;
}


